/**
 * @file
 * Javascript for collapser module.
 */

/**
 * Drupal API initialisation.
 */
Drupal.behaviors.collapser = function(context) {
  // Find all collapsible fieldsets in the current context and remember their
  // initial status. This is a workaround for D6 since we cannot
  // use the bindFirst event handler attachment method due to too old a
  // jQuery version in D6.
  $("fieldset.collapsible.collapser", context).each(function(){
    Drupal.collapser.statusData($(this), $(this).hasClass('collapsed'));
  })

  // Now process all standard FAPI fieldsets.

  // First bind an event handler to track every change in the collapsed state.
  $("form fieldset.collapsible > legend a:not(.collapser-processed)", context).each(function(){
    $(this).bind('click', function(event) {
      var fieldset = $(this).parents("fieldset").filter(':first');
      var cuid = Drupal.collapser.translateUid(fieldset.attr('id'));
      if (cuid) {
        // Toggle internal status and remember it.
        var status = Drupal.collapser.statusData(fieldset);
        Drupal.collapser.remember(cuid, status);
      }
    });
  }).addClass('collapser-processed');

  // Now process all standard FAPI fieldsets unless this has already happened on
  // the server (depending on user, role and permissions).
  if(!Drupal.settings.collapser.remoteSave) {
    $("form fieldset.collapsible.collapser:not(collapser-processed)").each(function(){
      var fieldset = $(this);
      var cuid = Drupal.collapser.translateUid(fieldset.attr('id'));
      if (cuid !== undefined) {
        var status = Drupal.collapser.cookie('Drupal.collapser.c' + cuid);
        // Only work if a cookie has ever been set.
        if (status !== undefined && status != fieldset.is('.collapsed')) {
          fieldset.find("legend a").filter(':first').click();
        }
      }
    }).addClass("collapser-processed");
  }
}

Drupal.collapser = {

  statuses: {},

  /**
   * Set or toggle an internally remembered fieldset status.
   *
   * @param domId
   *   The raw DOM id of the fieldset.
   *
   * @param status
   *   The initial status (optional); if omitted, status will be toggled.
   */
  statusData: function(object, status) {
    status = status === undefined ? !object.data('collapserStatus') : status;
    object.data('collapserStatus', status);
    return object.data('collapserStatus');
  },

  /**
   * Translate a collapser-uid-classname into a unique form/element ID.
   */
  translateUid: function(domClass) {
    uid = domClass.match(/\bcollapser-uid-([0-9]+)\b/);
    return typeof uid == 'object' ? uid[1] : false;
  },

  /**
   * Remember a fieldset status.
   *
   * @param cuid
   *   Unique collapser ID for the fieldset.
   * @param status
   *   Whether the fieldset was collapsed at click time.
   */
  remember: function(cuid, status) {
    status = status ? 1 : 0;
    if (Drupal.settings.collapser.remoteSave) {
      // 1 Save server-side for user based use.

      $.ajax({
        url: Drupal.settings.basePath + Drupal.settings.collapser.ajaxPath,
        error: function (a, b, c) {alert(b);},
        success: function (a, b, c) {},
        type: 'POST',
        data: {
          cuid: cuid,
          status: status
        }
      });
      // Ajax post to remote URL:
      // *Post raw data as above
      // *handle exceptions silently
    }
    else {
      // 2) Set a cookie for anonymous/client based use.
      Drupal.collapser.cookie('Drupal.collapser.c' + cuid, status, Drupal.settings.basePath, 365);
    }
  },

  /**
   * Cookie helper.
   *
   * jQuery.cookie is not available in D6. Left to our own devices.
   *
   * @param name
   * @param value
   * @param path
   * @param expires
   *   The expiration value in days from now.
   */
  cookie: function(name, value, path, expires) {
    // Generic cookie function needs seconds, not days for expire.
    // This is other than in jQuery.cookie.
    expires = expires * 60 * 60 * 24;
    // Do nothing if no name is given.
    if (name !== undefined && name != '') {
      // Set the cookie, if a value is given.
      if (value !== undefined) {
        if (path == '') {
          path = '/';
        }
        var exp = new Date();
        exp.setTime(exp.getTime() + expires);
        document.cookie = escape(name) + '=' + escape(value) + '; expires=' + exp.toGMTString() + ';path=' + path;
      }
      // Return the (new?) value of the cookie.
      if (document.cookie && document.cookie != '') {
        var re = new RegExp('(^|[; ])' + escape(name) + '=([^;]*)');
        values = document.cookie.match(re);
        if (values && values.length) {
          value = unescape(values[2]);
          return value;
        }
      }
    }
  }
};
